import { __decorate, __param } from 'tslib';
import { Inject, ElementRef, Renderer2, HostListener, Directive, NgModule } from '@angular/core';
import { DOCUMENT } from '@angular/common';
import { extractElementPosition } from 'ng-html-util';
import { polyfill } from 'smoothscroll-polyfill';

var ScrollSpyDirective_1;
let ScrollSpyDirective = ScrollSpyDirective_1 = class ScrollSpyDirective {
    // TODO: Change the any type to Document when fix https://github.com/angular/angular/issues/15640
    constructor(document, el, renderer) {
        this.document = document;
        this.el = el;
        this.renderer = renderer;
        this.elements = [];
        this.directNavigation = false;
    }
    ngAfterContentInit() {
        this.collectIds();
    }
    collectIds() {
        this.elements = [];
        let elements = this.el.nativeElement.querySelectorAll('a');
        for (let i = 0; i < elements.length; i++) {
            let elem = elements.item(i);
            let id = ScrollSpyDirective_1.getId(elem);
            if (!id)
                continue;
            let destination = this._getPeerElement(id);
            if (!destination)
                continue;
            elem.addEventListener('click', this._onLinkClicked.bind(this));
            this.elements.push({
                id,
                link: elem,
                destination
            });
        }
    }
    _onLinkClicked(event) {
        event.preventDefault();
        let target = event.currentTarget;
        let id = ScrollSpyDirective_1.getId(target);
        let destination = this._getPeerElement(id);
        this.directNavigation = true;
        let position = extractElementPosition(this.document, destination);
        window.scrollTo({ top: position.top - 25, left: 0, behavior: 'smooth' });
        this._cleanCurrentLink();
        this._setCurrentLink(target);
        this.directNavigation = false;
    }
    _getPeerElement(id) {
        let destination = this.document.getElementById(id);
        if (!destination)
            return null;
        return destination;
    }
    static getId(elem) {
        let href = elem.getAttribute('href');
        if (!href)
            return null;
        return href.replace('#', '');
    }
    onWindowScroll(event) {
        if (this.directNavigation)
            return;
        for (let elem of this.elements) {
            let top = elem.destination.getBoundingClientRect().top;
            if (top > 0 && top < 25) {
                this._cleanCurrentLink();
                this._setCurrentLink(elem.link);
                break;
            }
        }
    }
    _cleanCurrentLink() {
        if (!this.currentActiveLink)
            return;
        this.renderer.removeClass(this.currentActiveLink, 'active');
    }
    _setCurrentLink(elem) {
        this.currentActiveLink = elem;
        this.renderer.addClass(this.currentActiveLink, 'active');
    }
};
ScrollSpyDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: ElementRef },
    { type: Renderer2 }
];
__decorate([
    HostListener("window:scroll", ['$event'])
], ScrollSpyDirective.prototype, "onWindowScroll", null);
ScrollSpyDirective = ScrollSpyDirective_1 = __decorate([
    Directive({
        selector: '[scroll-spy]'
    }),
    __param(0, Inject(DOCUMENT))
], ScrollSpyDirective);

let ScrollSpyModule = class ScrollSpyModule {
    constructor() {
        polyfill();
    }
};
ScrollSpyModule = __decorate([
    NgModule({
        declarations: [ScrollSpyDirective],
        exports: [ScrollSpyDirective]
    })
], ScrollSpyModule);

/**
 * Generated bundle index. Do not edit.
 */

export { ScrollSpyDirective, ScrollSpyModule };
//# sourceMappingURL=ng-scroll-spy.js.map
