import { __values, __decorate, __param } from 'tslib';
import { Inject, ElementRef, Renderer2, HostListener, Directive, NgModule } from '@angular/core';
import { DOCUMENT } from '@angular/common';
import { extractElementPosition } from 'ng-html-util';
import { polyfill } from 'smoothscroll-polyfill';

var ScrollSpyDirective = /** @class */ (function () {
    // TODO: Change the any type to Document when fix https://github.com/angular/angular/issues/15640
    function ScrollSpyDirective(document, el, renderer) {
        this.document = document;
        this.el = el;
        this.renderer = renderer;
        this.elements = [];
        this.directNavigation = false;
    }
    ScrollSpyDirective_1 = ScrollSpyDirective;
    ScrollSpyDirective.prototype.ngAfterContentInit = function () {
        this.collectIds();
    };
    ScrollSpyDirective.prototype.collectIds = function () {
        this.elements = [];
        var elements = this.el.nativeElement.querySelectorAll('a');
        for (var i = 0; i < elements.length; i++) {
            var elem = elements.item(i);
            var id = ScrollSpyDirective_1.getId(elem);
            if (!id)
                continue;
            var destination = this._getPeerElement(id);
            if (!destination)
                continue;
            elem.addEventListener('click', this._onLinkClicked.bind(this));
            this.elements.push({
                id: id,
                link: elem,
                destination: destination
            });
        }
    };
    ScrollSpyDirective.prototype._onLinkClicked = function (event) {
        event.preventDefault();
        var target = event.currentTarget;
        var id = ScrollSpyDirective_1.getId(target);
        var destination = this._getPeerElement(id);
        this.directNavigation = true;
        var position = extractElementPosition(this.document, destination);
        window.scrollTo({ top: position.top - 25, left: 0, behavior: 'smooth' });
        this._cleanCurrentLink();
        this._setCurrentLink(target);
        this.directNavigation = false;
    };
    ScrollSpyDirective.prototype._getPeerElement = function (id) {
        var destination = this.document.getElementById(id);
        if (!destination)
            return null;
        return destination;
    };
    ScrollSpyDirective.getId = function (elem) {
        var href = elem.getAttribute('href');
        if (!href)
            return null;
        return href.replace('#', '');
    };
    ScrollSpyDirective.prototype.onWindowScroll = function (event) {
        var e_1, _a;
        if (this.directNavigation)
            return;
        try {
            for (var _b = __values(this.elements), _c = _b.next(); !_c.done; _c = _b.next()) {
                var elem = _c.value;
                var top_1 = elem.destination.getBoundingClientRect().top;
                if (top_1 > 0 && top_1 < 25) {
                    this._cleanCurrentLink();
                    this._setCurrentLink(elem.link);
                    break;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    ScrollSpyDirective.prototype._cleanCurrentLink = function () {
        if (!this.currentActiveLink)
            return;
        this.renderer.removeClass(this.currentActiveLink, 'active');
    };
    ScrollSpyDirective.prototype._setCurrentLink = function (elem) {
        this.currentActiveLink = elem;
        this.renderer.addClass(this.currentActiveLink, 'active');
    };
    var ScrollSpyDirective_1;
    ScrollSpyDirective.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
        { type: ElementRef },
        { type: Renderer2 }
    ]; };
    __decorate([
        HostListener("window:scroll", ['$event'])
    ], ScrollSpyDirective.prototype, "onWindowScroll", null);
    ScrollSpyDirective = ScrollSpyDirective_1 = __decorate([
        Directive({
            selector: '[scroll-spy]'
        }),
        __param(0, Inject(DOCUMENT))
    ], ScrollSpyDirective);
    return ScrollSpyDirective;
}());

var ScrollSpyModule = /** @class */ (function () {
    function ScrollSpyModule() {
        polyfill();
    }
    ScrollSpyModule = __decorate([
        NgModule({
            declarations: [ScrollSpyDirective],
            exports: [ScrollSpyDirective]
        })
    ], ScrollSpyModule);
    return ScrollSpyModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { ScrollSpyDirective, ScrollSpyModule };
//# sourceMappingURL=ng-scroll-spy.js.map
